/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

 /** ==========================================================================
 * 
 * @file		PersonDataListText.cpp --
 * @brief		Test class for person data list.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/PersonDataList.h"

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PersonDataListTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		airobots::PersonDataList original;
		original.setGenerator("test");
		original.setTimestamp(airobots::Timestamp(123, 456));
		
		airobots::PersonData data1;
		data1.setAngle(0.1);
		data1.setDistance(1000);
		data1.setFacing(false);
		data1.setGaze(1.0);
		data1.setIdentId(1);
		data1.setIdentName("Hans");
		data1.setHasNeighbour(false);
		data1.setNeighbourDistance(100);
		data1.setNeighbourId(-1);
		data1.setPoi(true);
		data1.setTalking(true);
		data1.setWalking(true);

		original.addPersonData(data1);

		airobots::PersonData data2;
		data2.setAngle(0.2);
		data2.setDistance(1200);
		data2.setFacing(true);
		data2.setGaze(2.0);
		data2.setIdentId(2);
		data2.setIdentName("Peter");
		data2.setHasNeighbour(true);
		data2.setNeighbourDistance(0.2);
		data2.setNeighbourId(4);
		data2.setPoi(false);
		data2.setTalking(false);
		data2.setWalking(false);

		original.addPersonData(data2);

		xmltio::Location originalLoc(airobots::PersonDataList::getTemplate(), airobots::PersonDataList::getBaseTag());
		originalLoc = original;	

		airobots::PersonDataList parsed = xmltio::extract<airobots::PersonDataList>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getPersonData().size(), original.getPersonData().size());

		airobots::PersonData parsed1 = parsed.getPersonData()[0];
		BOOST_REQUIRE_EQUAL(parsed1.isFacing(), data1.isFacing());
		BOOST_REQUIRE_EQUAL(parsed1.hasNeighbour(), data1.hasNeighbour());
		BOOST_REQUIRE_EQUAL(parsed1.isPoi(), data1.isPoi());
		BOOST_REQUIRE_EQUAL(parsed1.isTalking(), data1.isTalking());
		BOOST_REQUIRE_EQUAL(parsed1.isWalking(), data1.isWalking());
		BOOST_REQUIRE_CLOSE(parsed1.getAngle(), data1.getAngle(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed1.getDistance(), data1.getDistance(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed1.getGaze(), data1.getGaze(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed1.getNeighbourDistance(), data1.getNeighbourDistance(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed1.getIdentId(), data1.getIdentId());
		BOOST_REQUIRE_EQUAL(parsed1.getNeighbourId(), data1.getNeighbourId());
		BOOST_REQUIRE_EQUAL(parsed1.getIdentName(), data1.getIdentName());

		airobots::PersonData parsed2 = parsed.getPersonData()[1];
		BOOST_REQUIRE_EQUAL(parsed2.isFacing(), data2.isFacing());
		BOOST_REQUIRE_EQUAL(parsed2.hasNeighbour(), data2.hasNeighbour());
		BOOST_REQUIRE_EQUAL(parsed2.isPoi(), data2.isPoi());
		BOOST_REQUIRE_EQUAL(parsed2.isTalking(), data2.isTalking());
		BOOST_REQUIRE_EQUAL(parsed2.isWalking(), data2.isWalking());
		BOOST_REQUIRE_CLOSE(parsed2.getAngle(), data2.getAngle(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed2.getDistance(), data2.getDistance(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed2.getGaze(), data2.getGaze(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed2.getNeighbourDistance(), data2.getNeighbourDistance(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed2.getIdentId(), data2.getIdentId());
		BOOST_REQUIRE_EQUAL(parsed2.getNeighbourId(), data2.getNeighbourId());
		BOOST_REQUIRE_EQUAL(parsed2.getIdentName(), data2.getIdentName());

	} catch (std::exception & e) {
		BOOST_FAIL(std::string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testLocationReuse) {

	const std::string attributeName = "testattribute";
	const std::string elementName = "ADDITIONAL";

	airobots::PersonDataList original;
	original.setGenerator("test");
	original.setTimestamp(airobots::Timestamp(123, 456));
		
	airobots::PersonData data1;
	data1.setAngle(0.1);
	data1.setDistance(1000);
	data1.setFacing(true);
	data1.setGaze(1.0);
	data1.setIdentId(1);
	data1.setIdentName("Hans");
	data1.setHasNeighbour(false);
	data1.setNeighbourDistance(0.1);
	data1.setNeighbourId(0.2);
	data1.setPoi(false);
	data1.setTalking(false);
	data1.setWalking(false);

	original.addPersonData(data1);

	airobots::PersonData data2;
	data2.setAngle(0.2);
	data2.setDistance(1200);
	data2.setFacing(true);
	data2.setGaze(2.0);
	data2.setIdentId(2);
	data2.setIdentName("Peter");
	data2.setHasNeighbour(false);
	data2.setNeighbourDistance(0.2);
	data2.setNeighbourId(0.3);
	data2.setPoi(false);
	data2.setTalking(false);
	data2.setWalking(false);

	original.addPersonData(data2);

	xmltio::Location originalLoc(airobots::PersonDataList::getTemplate(), airobots::PersonDataList::getBaseTag());
	originalLoc = original;

	std::cout << "adding additional attribute and elements" << std::endl;
	originalLoc[attributeName.c_str()] = 23;
	xmltio::Location additionalElement("<" + elementName + "/>", elementName);
	originalLoc.add(additionalElement);
	std::cout << "changed document" << std::endl;

	std::cout << originalLoc.getDocumentText(true);

	// do some parsing and re-serialization into the old location
	airobots::PersonDataList parsedFirstTime = xmltio::extract<airobots::PersonDataList>(originalLoc);
	originalLoc = parsedFirstTime; 

	airobots::PersonDataList parsedSecondTime = xmltio::extract<airobots::PersonDataList>(originalLoc);
	originalLoc = parsedSecondTime;

	// ensure that the parsed data is still the same and additional elements in
	// the xml still exist
	BOOST_REQUIRE_EQUAL(original.size(), parsedSecondTime.size());
	originalLoc[xmltio::XPath("@" + attributeName)];
	originalLoc[xmltio::XPath(elementName)];

}

BOOST_AUTO_TEST_SUITE_END()
