/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.asu;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

/**
 * Contains all categories of utterances.
 * 
 * @author malinke
 */
public enum UtteranceCategory {

	/**
	 * Instruction.
	 */
	instruction("task"),
	/**
	 * Description.
	 */
	description("description"),
	/**
	 * Socialisation.
	 */
	socialisation("greeting", "parting", "socialisation"),
	/**
	 * Query.
	 */
	query("question"),
	/**
	 * Object.
	 */
	object("object"),
	/**
	 * Grip.
	 */
	grip("grip"),
	/**
	 * Deletion.
	 */
	deletion("deletion"),
	/**
	 * Confirmation.
	 */
	confirmation("confirmation"),
	/**
	 * Negation.
	 */
	negation("negation"),
	/**
	 * Serves as fallback category. Will be returned, if no other is found.
	 */
	fragment();

	private static Map<String, UtteranceCategory> catMap;

	private final String[] frameNames;

	private UtteranceCategory(final String... frameNames) {
		this.frameNames = frameNames;
	}

	/**
	 * Returns the category of a frame.
	 * 
	 * @param frameName
	 *            the name of the frame
	 * @return fitting category
	 */
	public static UtteranceCategory categoryOf(final String frameName) {
		if (catMap == null) {
			buildMap();
		}
		final String lowerName = frameName.toLowerCase();
		if (catMap.containsKey(lowerName)) {
			return catMap.get(lowerName);
		}
		Logger.getLogger(UtteranceCategory.class.getName()).severe(
				"No category for " + frameName + ". --> fragment.");
		return fragment;
	}

	private static void buildMap() {
		catMap = new HashMap<String, UtteranceCategory>();
		for (UtteranceCategory c : values()) {
			for (String frameName : c.frameNames) {
				catMap.put(frameName, c);
			}
		}
	}

}
